@@ -327,11 +327,12 @@ class LabNotebookTests(unittest.TestCase):
 
     def testDeleteResultSetByTag(self):
         '''Test we can delete a result set.'''
+        rs = self._nb.addResultSet('first')
         rs = self._nb.addResultSet('second')
-        self._nb.deleteResultSet(LabNotebook.DEFAULT_RESULTSET)
-        self.assertEqual(len(self._nb), 1)
+        self._nb.deleteResultSet('first')
+        self.assertEqual(len(self._nb), 2)
         self.assertEqual(self._nb.currentTag(), 'second')
-        self.assertCountEqual(self._nb.resultSets(), ['second'])
+        self.assertCountEqual(self._nb.resultSets(), ['second', LabNotebook.DEFAULT_RESULTSET])
         self.assertEqual(self._nb.current(), rs)
 
     def testDeleteResultSetByReference(self):
@@ -349,6 +350,11 @@ class LabNotebookTests(unittest.TestCase):
         with self.assertRaises(Exception):
             self._nb.deleteResultSet(rs)
 
+    def testDeleteDefault(self):
+        '''Test we can't delete the default result set.'''
+        with self.assertRaises(Exception):
+            self._nb.deleteResultSet(LabNotebook.DEFAULT_RESULTSET)
+
     def testKeys(self):
         '''Test we can extract the result set tags.'''
         self.assertCountEqual(self._nb.keys(), [LabNotebook.DEFAULT_RESULTSET])
@@ -427,7 +433,6 @@ class LabNotebookTests(unittest.TestCase):
         p = self._nb.pendingResultParameters('1234')
         self.assertCountEqual(rc1[Experiment.PARAMETERS], p)
 
-
 # TODO: Test we can add metadata
 
 if __name__ == '__main__':
