@@ -111,6 +111,13 @@ class ResultSetTests(unittest.TestCase):
         self.assertCountEqual(self._rs.metadataNames(), Experiment.StandardMetadata.union(set(['additional'])))
         self.assertEqual(dtype.fields['additional'][0], numpy.dtype(ResultSet.TypeMapping[bool]))
 
+        # check we can now extend the
+        #  metadata
+        self._rc[Experiment.METADATA]['andagain'] = 'here we go'
+        dtype = self._rs.inferDtype(self._rc)
+        self.assertCountEqual(self._rs.metadataNames(), Experiment.StandardMetadata.union(set(['additional', 'andagain'])))
+        self.assertEqual(dtype.fields['andagain'][0], numpy.dtype(ResultSet.TypeMapping[str]))
+
     def testAddResults(self):
         '''Test we can add more results.'''
 
@@ -196,7 +203,7 @@ class ResultSetTests(unittest.TestCase):
         self._rc[Experiment.RESULTS]['extra'] = 'hello'
         self._rs.addSingleResult(self._rc)
         df = self._rs.dataframe()
-        self.assertTrue((df[df['k'] == 1]['extra'] == 0).all())
+        self.assertTrue((df[df['k'] == 1]['extra'] == '').all())
         self.assertTrue((df[df['k'] == 2]['extra'] == 'hello').all())
         self.assertTrue(self._rs.isDirty())
 
@@ -322,7 +329,7 @@ class ResultSetTests(unittest.TestCase):
         self._rs.addSingleResult(self._rc)
         df = self._rs.dataframe()
         self.assertTrue((df[df['singleton'] == 3]['radically'] == 'wrong').all())
-        self.assertTrue((df[df['singleton'] != 3]['radically'] == 0).all())
+        self.assertTrue((df[df['singleton'] != 3]['radically'] == '').all())
  
     def testSingleResult(self):
         '''Test retrieval of a single result.'''
@@ -356,7 +363,7 @@ class ResultSetTests(unittest.TestCase):
         df = self._rs.dataframeFor(params1)
         self.assertEqual(len(df.index), 1)
         self.assertTrue((df[df['singleton'] == params1['singleton']]['first'] == 1).all())
-        self.assertTrue((df[df['singleton'] == params1['singleton']]['radically'] == 0).all())
+        self.assertTrue((df[df['singleton'] == params1['singleton']]['radically'] == '').all())
         self.assertTrue((df[df['singleton'] == 3]['first'] == 1).all())
         self.assertTrue((df[df['singleton'] == 3]['radically'] == 'wrong').all())
 
@@ -511,10 +518,36 @@ class ResultSetTests(unittest.TestCase):
         self._rs.addSingleResult(self._rc)
         self._rs.cancelSinglePendingResult('1234')
         self.assertCountEqual(self._rs.pendingResults(), [])
+
+        # check we have the cancelled result still
         rss = self._rs.dataframeFor(self._rc[Experiment.PARAMETERS])
+        self.assertEqual(len(rss), 2)
+
+        # check there's only one successful result
+        rss = self._rs.dataframeFor(self._rc[Experiment.PARAMETERS], only_successful=True)
         self.assertEqual(len(rss), 1)
         self.assertEqual(rss['first'][0], 1)
 
+    def testPendingResultsFor(self):
+        '''Test we can retrieve pending results for a sub-set of parameters.'''
+        params = dict()
+        params['a'] = 10
+        params['b'] = 50
+        params['c'] = 'fifty'
+        self.assertEqual(len(self._rs.pendingResultsFor(params)), 0)
+        self._rs.addSinglePendingResult(params, '1234')
+        self.assertEqual(len(self._rs.pendingResultsFor(params)), 1)
+        params['b'] = 90
+        params['c'] = 'ninety'
+        self._rs.addSinglePendingResult(params, '5678')
+        params['b'] = 90
+        params['c'] = 'ninety'
+        self._rs.addSinglePendingResult(params, '91011')
+        self.assertEqual(len(self._rs.pendingResultsFor(dict(a=10))), 3)
+        self.assertEqual(len(self._rs.pendingResultsFor(dict(b=50))), 1)
+        self.assertEqual(len(self._rs.pendingResultsFor(dict(a=10, b=50))), 1)
+        self.assertEqual(len(self._rs.pendingResultsFor(dict(a=15, b=50))), 0)
+
     def testNumberOfPendingResultsZero(self):
         '''Test we can handle zero pending results.'''
         self.assertEqual(self._rs.numberOfPendingResults(), 0)        
@@ -524,6 +557,14 @@ class ResultSetTests(unittest.TestCase):
         self._rs.addSinglePendingResult(self._rc[Experiment.PARAMETERS], '1234')
         self.assertEqual(self._rs.numberOfPendingResults(), 1)        
 
+    def testJobId(self):
+        '''Test we raise exceptions for unrecognised job ids.'''
+        self._rs.addSinglePendingResult(self._rc[Experiment.PARAMETERS], '1234')
+        with self.assertRaises(PendingResultException):
+            self._rs.cancelSinglePendingResult('4567')
+        with self.assertRaises(PendingResultException):
+            self._rs.resolveSinglePendingResult('4567')
+
     def testDuplicateJobs(self):
         '''Test we catch addition of duplicate job ids.'''
         self._rs.addSinglePendingResult(self._rc[Experiment.PARAMETERS], '1234')
@@ -558,9 +599,15 @@ class ResultSetTests(unittest.TestCase):
         self._rs.cancelSinglePendingResult('4567')
         self.assertCountEqual(self._rs.pendingResults(), [])
 
+        # check the two cancelled results were zeroed correctly
         rss = self._rs.dataframeFor(self._rc[Experiment.PARAMETERS])
-        self.assertEqual(len(rss), 2)
-        self.assertCountEqual(rss['first'], [ 1, 2 ])
+        self.assertEqual(len(rss), 4)
+        self.assertCountEqual(rss['first'], [ 0, 0, 1, 2 ])
+
+        # check that we project out the right successes
+        rsss = rss[rss[Experiment.STATUS] == True]
+        self.assertEqual(len(rsss), 2)
+        self.assertCountEqual(rsss['first'], [ 1, 2 ])
 
     def testPendingParameterNames(self):
         '''Test we have to respect parameter names when adding pending jobs.'''
@@ -585,9 +632,16 @@ class ResultSetTests(unittest.TestCase):
         self._rs.addSinglePendingResult(self._rc[Experiment.PARAMETERS], '4567')
         self.assertCountEqual(self._rs.pendingResults(), [ '1234', '4567' ])
 
+        # check job gets cancelled
         self._rs.cancelSinglePendingResult('1234')
         self.assertCountEqual(self._rs.pendingResults(), [ '4567' ])
 
+        # check the exception is correct
+        rcs = self._rs.resultsFor(dict(duplicate=10))
+        self.assertEqual(len(rcs), 1)
+        rc = rcs[0]
+        self.assertTrue(isinstance(rc[Experiment.METADATA][Experiment.EXCEPTION], CancelledException))
+
     def testFailedResult(self):
         '''Test we can save a failed result, i.e., with an exception and no results.'''
         self._rc[Experiment.METADATA][Experiment.STATUS] = False
@@ -656,7 +710,7 @@ class ResultSetTests(unittest.TestCase):
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params1), rc1))
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params2), rc2))
         self.assertEqual(len(self._rs.dataframeFor(params3)), 0)
-        self.assertEqual(len(self._rs.results()), 2)
+        self.assertEqual(len(self._rs.results()), 3)
         self.assertCountEqual(self._rs.pendingResults(), [ '3' ])
         self.assertFalse(self._rs.ready())
 
@@ -664,8 +718,9 @@ class ResultSetTests(unittest.TestCase):
         self._rs.cancelSinglePendingResult('3')
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params1), rc1))
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params2), rc2))
-        self.assertEqual(len(self._rs.dataframeFor(params3)), 0)
-        self.assertEqual(len(self._rs.results()), 2)
+        self.assertEqual(len(self._rs.dataframeFor(params3)), 1)
+        self.assertEqual(len(self._rs.dataframeFor(params3, only_successful=True)), 0)
+        self.assertEqual(len(self._rs.results()), 4)
         self.assertCountEqual(self._rs.pendingResults(), [])
         self.assertTrue(self._rs.ready())
 
@@ -673,8 +728,8 @@ class ResultSetTests(unittest.TestCase):
         self._rs.addSinglePendingResult(params3, '3')
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params1), rc1))
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params2), rc2))
-        self.assertEqual(len(self._rs.dataframeFor(params3)), 0)
-        self.assertEqual(len(self._rs.results()), 2)
+        self.assertEqual(len(self._rs.dataframeFor(params3)), 1)
+        self.assertEqual(len(self._rs.results()), 4)
         self.assertCountEqual(self._rs.pendingResults(), [ '3' ])
         self.assertFalse(self._rs.ready())
 
@@ -682,11 +737,46 @@ class ResultSetTests(unittest.TestCase):
         self._rs.cancelSinglePendingResult('3')
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params1), rc1))
         self.assertTrue(self._resultsEqual(self._rs.dataframeFor(params2), rc2))
-        self.assertEqual(len(self._rs.dataframeFor(params3)), 0)
-        self.assertEqual(len(self._rs.results()), 2)
+        self.assertEqual(len(self._rs.dataframeFor(params3)), 2)
+        self.assertEqual(len(self._rs.results()), 5)
         self.assertCountEqual(self._rs.pendingResults(), [])
         self.assertTrue(self._rs.ready())
 
+    def testLocking(self):
+        '''Test we can lock result sets.'''
+        e = SampleExperiment()
+        
+        # create three results
+        params1 = dict(a  = 1, b = 2)
+        rc1 = e.set(params1).run()
+        params2 = dict(a  = 10, b = 12)
+        rc2 = e.set(params2).run()
+        params3 = dict(a  = 45, b = 11)
+        rc3 = e.set(params3).run()
+
+        # add first result, with second pending
+        self.assertFalse(self._rs.isLocked())
+        self._rs.addSingleResult(rc1)
+        self._rs.addSinglePendingResult(params2, '1234')
+
+        # finish and lock the result set
+        self._rs.finish()
+        self.assertEqual(self._rs.numberOfResults(), 2)
+        self.assertEqual(self._rs.numberOfPendingResults(), 0)
+        self.assertTrue(self._rs.isLocked())
+
+        # make sure we can't add further jobs or otherwise change the result set
+        with self.assertRaises(ResultSetLockedException):
+            self._rs.addSingleResult(rc3)
+        with self.assertRaises(ResultSetLockedException):
+            self._rs.addSinglePendingResult(rc3, '3456')
+        with self.assertRaises(ResultSetLockedException):
+            self._rs['attribute'] = 'not saved'
+
+        # make sure we still have access
+        self.assertEqual(len(self._rs.results()), 2)
+
+
 if __name__ == '__main__':
     unittest.main()
 
